@@ -46,17 +46,17 @@
 /*
  * Major version.
  */
-#define	KCGI_VMAJOR	0
+#define	KCGI_VMAJOR	1
 
 /*
  * Minor version.
  */
-#define	KCGI_VMINOR	13
+#define	KCGI_VMINOR	0
 
 /*
  * Build version.
  */
-#define	KCGI_VBUILD	4
+#define	KCGI_VBUILD	0
 
 /*
  * Version string of major.minor.build (as a literal string).
@@ -152,9 +152,14 @@ enum	krequ {
 	KREQU_IF_UNMODIFIED_SINCE,
 	KREQU_MAX_FORWARDS,
 	KREQU_ORIGIN,
+	KREQU_PRIORITY,
 	KREQU_PROXY_AUTHORIZATION,
 	KREQU_RANGE,
 	KREQU_REFERER,
+	KREQU_SEC_FETCH_DEST,
+	KREQU_SEC_FETCH_MODE,
+	KREQU_SEC_FETCH_USER,
+	KREQU_SEC_FETCH_SITE,
 	KREQU_USER_AGENT,
 	KREQU__MAX
 };
@@ -187,6 +192,7 @@ enum	kresp {
 	KRESP_LOCATION,
 	KRESP_P3P,
 	KRESP_PRAGMA,
+	KRESP_PRIORITY,
 	KRESP_PROXY_AUTHENTICATE,
 	KRESP_REFRESH,
 	KRESP_RETRY_AFTER,
@@ -451,6 +457,8 @@ struct	kreq {
 	struct khead		 *reqmap[KREQU__MAX];
 	struct khead		 *reqs;
 	size_t		 	  reqsz;
+	struct khead		 *envs;
+	size_t		 	  envsz;
 	enum kmethod		  method;
 	enum kauth		  auth;
 	struct khttpauth	  rawauth;
